#!/usr/bin/env python

# Copyright 2008-2013 by Tim Bird

# turn this on to show the game data in the admin view
# (for debugging)
print_data = 0 

num_categories = 5

import sys, os, cgi

import cgitb ; cgitb.enable()

data_dir = "/home/game/pdata/"
game_file_pattern = "perildata-([0-9][0-9][0-9]).txt"
game_file_fmt = "perildata-%03d.txt"

# import the answer data
sys.path.append(data_dir)
from answers import *

# the answers.py file has the answer data for each round:
# The data has the format:
# answers = [] [ <# round 1 data>
#	[category_name1, answer1-1, answer1-2, answer1-3, answer1-4, answer1-5]
#	[category_name2, answer2-1, answer2-2, answer2-3, answer2-4, answer2-5]
#       ...
# ] [ <#round 2 data> ... ]
# final = [description, answer]
# super_peril = [(round, category, question_num),...]
#
# The game progresses as a series of rounds.  round1 = first game board
# there are two possible views of the game, the player view.  If player_view=1,
# the the player_view is shown, an no actions are processed.
# if answer_to_show is not zero, then an answer is shown in player_view
#
# The player view automatically refreshes, so as the game state changes,
# it should change without manual intervention.
#
# The admin screen allows controlling the question selection, and giving
# or subtracting points.
# 

stub_data = {
"data_sequence":"0",
"round":"1",
"answer_to_show":"0",
"wager":"0",
"score_a":"0",
"score_b":"0",
"score_c":"0",
"player_a_name":"Player A",
"player_b_name":"Player B",
"player_c_name":"Player C",
"cat_0_done":"000000",
"cat_1_done":"000000",
"cat_2_done":"000000",
"cat_3_done":"000000",
"cat_4_done":"000000",
"cat_5_done":"000000",
"cat_6_done":"000000",
}

######################################################

def read_game_data_from_last_file():
	# read from the last-numbered game-data file
	import os, re
	file_list = os.listdir(data_dir)
	max_sequence = -1
	max_filename = "unknown"
	for filename in file_list:
		m = re.match(game_file_pattern, filename)
		if m:
			sequence = int(m.groups()[0])
			if sequence > max_sequence:
				max_sequence = sequence
				max_filename = filename 
		
	last_game_filename = data_dir + max_filename
	return read_game_data(last_game_filename)

def read_game_data(game_filename):
	try:
		game_lines = open(game_filename, "r").readlines()
	except:
		print "Warning: failed to open game data file: %s" % game_filename
		game_lines = []

	data = stub_data.copy()
	for line in game_lines:
		line = line.strip()
		if line and line[0]!='#':
			(name, value) = line.split('=',1)
			#print "data[%s]=%s<br>" % (name, value)
			data[name] = value

	data["game_filename"] = game_filename
	return data

######################################################

def write_game_data(data):
	# write out file to next filename in sequence

	# increment sequence number
	data_sequence = int(data["data_sequence"])
	data_sequence += 1
	data["data_sequence"] = str(data_sequence)
	game_filename = data_dir+game_file_fmt % data_sequence
	data["game_filename"] = game_filename

	fd = open(game_filename, "w")
	klist = data.keys()
	klist.sort()
	for name in klist:
		fd.write("%s=%s\n" % (name, data[name]))
	fd.close()

######################################################

def remove_undo_data_files():
	# scan for game-data files
	import os, re
	file_list = os.listdir(data_dir)
	for filename in file_list:
		m = re.match(game_file_pattern, filename)
		if m:
			target = data_dir + filename
			os.unlink(target)

######################################################

def question_value(data, row):
	row = int(row)
	round = int(data["round"])
	return row * (round * 100)

######################################################


# show the game board
# big_board indicates whether to use large fonts and fill the screen

# here is some size data
big_screen_data = {
	"border":0,
	"debug":0,
	"size":5,
	"tsize":7,
	"fsize":6,
	# main gameboard offsets
	"w0":100,
	"h0":30,
	# main gameboard cell dimensions
	"w":200,
	#"h":80,
	"h":80,
	"cat_font":5,
	"answer_height":440,
	"value_font_size":5,
	# score table stuff
	"score_offset":300,
	"score_spacing":200,
}

small_screen_data = {
	"border":0,
	"debug":0,
	"size":3,
	"tsize":3,
	"fsize":3,
	# main gameboard offsets
	"w0":1,
	"h0":1,
	# main gameboard cell dimensions
	"w":60,
	"h":60,
	"cat_font":3,
	"answer_height":200,
	"value_font_size":3,
	# score table stuff
	"score_offset":100,
	"score_spacing":40,
}

######################################################

def show_cell(data, row, col):
	round = int(data["round"])
	width = data["w"]
	done = data["cat_%d_done" % col][row]

	if row==0:
		category = answers[round][col][0]
		cat_font = data["cat_font"]
		print """<td width="%s" align="center"><b><font size="%s">%s</font></b></td>""" % (width, cat_font, category)
	else:
		value = question_value(data, row)
		if done=="1":
			value = ""
			
		print """<td width="%s" align="center"><b><font size="%s">
	<a href="peril.cgi?action=show_answer&row=%d&col=%d">
	%s </b>
</td>""" % (width, "5", row, col, value)

######################################################
#
# state machine is:
# round = 1 -> 1st round board
# round = 2 -> 2nd round board
# round = 3 -> show final category
# round = 4 -> show final answer
# (edit data to add wagers and answers)
# round = 5 -> link to final_form (player_view=show final answer)
# round = 6 -> reveal final player questions
# round = 7 -> show winner

def show_final(data, round):
	answer_height = int(data["answer_height"])

	if round==3:
		print """<table width="100%%" border=1>
<tr height="%s"><td width="5%%"></td>
<td valign=middle align=center width="90%%"><font size="9">%s</font></td>
<td width="5%%"></td>
</tr></table>""" % (answer_height, final[0])

	if round==4:
		print """<table width="100%%" border=1>
<tr height="%s"><td width="5%%"></td>
<td valign=middle align=center width="90%%"><font size="9">%s</font></td>
<td width="5%%"></td>
</tr></table>""" % (answer_height, final[1])

	if round==5:
		if player_view:
			print """<table width="100%%" border=1>
<tr height="%s"><td width="5%%"></td>
<td valign=middle align=center width="90%%"><font size="9">%s</font></td>
<td width="5%%"></td>
</tr></table>
Now Entering your Final Peril data""" % (answer_height, final[1])
		else:
			print """<table width="100%%" border=1>
<tr height="%s"><td width="5%%"></td>
<td valign=middle align=center width="90%%"><font size="9">Please enter final wages and answers by clicking on this link:<br>
<a href="peril.cgi?action=final_form">Final Data Entry</a></font></td>
<td width="5%%"></td>
</tr></table>""" % (answer_height)

	if round==6:
		show_final_result(data)

	if round==7:
		show_winner(data)

######################################################
# OK, this is a bit complicated
# After receiving the inputs for each player (their question and wager)
# for the final round, this routine displays their information, at first
# hidden.  Each question and wager is revealed, and if the question was
# correct, the score is updated for that player
#
# some data variables used: hide_question_<player>,
# question_result_<player> (can be: "unknown", "correct", "wrong")
# hide_wager_<player>
#
# display sequence is:
#   reveal question form (if hide_question_<player> ==1 )
#   last result form (if hide_question_<player>==0, and question_result=="unknown")
#   reveal wage form (if question_result!="unknown" and hide_wager_<player>==1)
#   score it link (if question_result!="unknown" and hide_wager_<player>=="0")
#
def show_final_result(data):
	# determine which values to reveal
	data_copy = {}
	data_copy.update(data)
	answer_height = int(data["answer_height"])

	for player in ["a", "b", "c"]:
		for field in ["wager", "question"]:
			hide_str = "hide_"+field+"_"+ player
			hide_flag = data[hide_str]
			if hide_flag == "1":
				data_copy[field+"_"+player] = "XXXXXXXX"

	print """
<table width="100%%" border=1>
<tr height=1><td rowspan=4 width="5%%"></td>
<td width="90%%"><td rowspan=4 width="5%%"></td></tr>
""" % data

	for player in ["a", "b", "c"]:
		# figure out question status, if known
		result = data["question_result_"+player]
		if result=="unknown":
			status_str = ""
		else:
			status_str = "(status: %s)" % result

		format = """
<tr height="%s">
<td>
  <table width="100%%%%"><tr>
    <td width="80%%%%">
      <font size="%%(tsize)s">%%(player_%s_name)s</font></td>
    <td rowspan=2 align=right>
      <table border=1 bordercolor=black><tr><td align=right bgcolor=white>
	<font size=%%(fsize)s>&nbsp;&nbsp;%%(score_%s)s&nbsp;&nbsp;</font></td></tr>
      </table>
  </tr><tr>
    <td><font size="%%(fsize)s">Question: %%(question_%s)s %s</font></td>
  </tr><tr>
    <td><font size="%%(fsize)s">Wager:</a> %%(wager_%s)s</font></td>
  </tr><tr>
""" % (answer_height/3, player, player, player,    status_str,player)

######                  <name>, <score>,<question>,<status>, <wager> ########

		print format % data_copy

		if player_view:
			print """	</tr></table>\n</td></tr>"""
			continue
		
		print "</tr><tr><td>"
		
		# in admin view, decide which link we're showing

		if data["hide_question_"+player] == "1":
			print """
    <a href="peril.cgi?action=reveal&field=question&player=%s">Reveal question</a>
""" % (player)
		if data["hide_question_"+player] == "0" and \
			data["question_result_"+player] == "unknown":
			print """
<a href="peril.cgi?action=set_last_question_result&player=%s&result=correct">Correct?</a>
<a href="peril.cgi?action=set_last_question_result&player=%s&result=wrong">Wrong?</a>
""" % (player, player)

		if data["question_result_"+player] != "unknown" and \
			data["hide_wager_"+player]=="1":
			print """
    <a href="peril.cgi?action=reveal&field=wager&player=%s">Reveal wager</a>
""" % (player)

		if data["hide_wager_"+player]=="0" and \
			data["last_score_recorded_"+player]=="0":
			print """
<a href="peril.cgi?action=last_score&player=%s">Score it!</a>
""" % (player)

		print """	</td></tr></table>\n</td></tr>"""

	# end of for loop over players
	print """</tr></table></font>"""

######################################################

def show_winner(data):
	color = "LightGreen"
	answer_height = int(data["answer_height"])
	value_font_size = int(data["value_font_size"])

	# calculate the winner
	score_a = int(data["score_a"])
	score_b = int(data["score_b"])
	score_c = int(data["score_c"])
	winner_name = data["player_a_name"]
	winner_score = score_a

	tie = 0

	if score_b > winner_score:
		winner_name = data["player_b_name"]
		winner_score = score_b
	else:
		# check for a tie
		if score_b == winner_score:
			winner_name += " and "+data["player_b_name"]
			tie = 1

	if score_c > winner_score:
		winner_name = data["player_c_name"]
		winner_score = score_c

	else:
		if score_c == winner_score:
			winner_name += " and "+data["player_c_name"]
			tie = 1

	winner_str = "The winner is"
	if tie:
		winner_str = "The winners are"

	print """<table width="100%%" bgcolor="%s" border=1>
<tr height="%s"><td colspan=3 valign=middle align=center width=100%%>
<font size="9">%s<br>%s</font>
</td></tr>
<tr><td colspan=3 align=center>
<font size="%s">With a score of: %d</font>
</td></tr>
""" % (color, answer_height, winner_str, winner_name, value_font_size, winner_score) 

######################################################

def show_board(data, player_view):
	round = int(data["round"])
	answer_to_show = data["answer_to_show"]
	if answer_to_show=="0":
		answer_to_show = 0

	# get size data for display
	if player_view:
		size_data = big_screen_data
	else:
		size_data = small_screen_data

	data.update(size_data)

	# offset table in browser frame, and establish horizontal spacings
	print """
<table border=%(border)s bgcolor="orange" cellspacing=0 cellpadding=0>
<tr bgcolor="LightBlue">
<td width="%(w0)s" rowspan=8></td>
<td height="%(h0)s" width="%(w)s"></td>
	""" % data
	
	for i in range(0,num_categories-1):
		print """<td width="%(w)s"></td>""" % data

	print """
<td width="%(w0)s" rowspan=8></td>
</tr>""" % data

	# title
	title = "Peril"
	if round==2: title = "Double Peril"
	if round>=3: title = "Final Peril"
	data["title"] = title
	print """
<tr bgcolor="LightBlue">
<td colspan=7 align=center><b><font size="%(tsize)s" color="crimson">
%(title)s</font></font></td>
</tr>""" % data

	# show round and multiplier information
	print "</font></td></tr>" % data


	# use this to debug columns
	if data["debug"]==1:
		print """<tr bgcolor="LightBlue">"""
		for i in range(1, num_categories+1):
			print """<td align=center>%d</td>""" % i
	
		print "</tr>"

	# show main board data
	if round<3:
		if answer_to_show:
			show_answer(data)
		else:
			# print table for normal round
			for row in range(0,6):
				print """<tr height="%(h)s">""" % data
				for col in range(0,num_categories):
					show_cell(data, row, col)
				print "</tr>"

	if round>=3:
		show_final(data, round)
		print "</table>"
		html_end()
		sys.exit(0)

	# end of main gameboard table
	print "</table>"

	# scores
	print"""
<table border=%(border)s bgcolor="orange" cellspacing=0 cellpadding=0>
<tr bgcolor="LightBlue">
<td width="%(score_offset)s"></td>
<td align=center><font size=%(size)s><b>%(player_a_name)s</b></font>
  <table border=1 bordercolor=black><tr><td align=right bgcolor=white>
	<font size=%(size)s>&nbsp;&nbsp;%(score_a)s&nbsp;&nbsp;</font></td></tr>
  </table>
</td>

<td width="%(score_spacing)s"></td>

<td align=center><font size=%(size)s><b>%(player_b_name)s</b></font>
  <table border=1 bordercolor=black><tr><td align=right bgcolor=white>
	<font size=%(size)s>&nbsp;&nbsp;%(score_b)s&nbsp;&nbsp;</font></td></tr>
  </table>
</td>

<td width="%(score_spacing)s"></td>

<td align=center><font size=%(size)s><b>%(player_c_name)s</b></font>
  <table border=1 bordercolor=black><tr><td align=right bgcolor=white>
	<font size=%(size)s>&nbsp;&nbsp;%(score_c)s&nbsp;&nbsp;</font></td></tr>
  </table>
</td>
</tr>
</table>""" % data

	# clean up data
	for key in size_data.keys():
		del data[key]

	# end of show_board()

######################################################

def show_super_wager_form(data, row, col):
	global player_view

	if player_view:
		wager_form = "Please indicate your wager."
	else:
		wager_form = """
<FORM method=post action="peril.cgi">
<INPUT type=hidden name="action" value="set_super_wager">
<INPUT type=hidden name="row" value="%s">
<INPUT type=hidden name="col" value="%s">
Wager: <INPUT type=text name="wager">
<input type="Submit" name="submit" value="Submit">
""" % (row, col)

	color="red"
	answer_height=data["answer_height"]

	print """<table width="100%%" bgcolor="%s" border=1>
<tr height="%s"><td colspan=3 valign=middle align=center width=100%%>
<font size="9">SUPER<br>PERIL</font>
</td></tr>
<tr><td colspan=3 align=center>
%s
<table border=0>
</td></tr>
""" % (color, answer_height, wager_form)

######################################################

def show_answer(data):
	answer_height = int(data["answer_height"])
	answer_to_show = data["answer_to_show"]
	value_font_size = int(data["value_font_size"])

	wager = int(data["wager"])
	row, col = answer_to_show.split(":")
	round = int(data["round"])
	answer = answers[round][int(col)][int(row)]
	value = question_value(data, row)
	value_type_str = "Value"

	if wager:
		value = wager
		value_type_str = "Wager"

	# check for super_peril
	super_flag = 0
	for i in range(len(super_peril)):
		sp_tuple = super_peril[i]
		if sp_tuple[0]==round and sp_tuple[1]-1==int(col) and \
			sp_tuple[2]==int(row):
			super_flag = 1

	color = "LightBlue"

	# FIXTHIS - should I show the wager form in player_view mode???
	# could check here and skip
	if super_flag and not wager:
		show_super_wager_form(data, row, col)
		return
	
	if super_flag:
		color = "orange"
		
	print """<table width="100%%" bgcolor="%s" border=1>
<tr height="%s"><td colspan=3 valign=middle align=center width=100%%><font size="9">%s</font>
</td></tr>
<tr><td colspan=3 align=center>
<font size="%s">%s is: %d</font>
</td></tr>
""" % (color, answer_height, answer, value_font_size, value_type_str, value)

	if player_view:
		return

	# show admin view for this answer
	data["row"] = row
	data["col"] = col
	data["value"] = value

	
	print """
<tr height=1><td></td></tr>
<tr>
<td width="30%%">
<a href="peril.cgi?action=a_correct&row=%(row)s&col=%(col)s&value=%(value)s">%(player_a_name)s right?</a><br>
<a href="peril.cgi?action=b_correct&row=%(row)s&col=%(col)s&value=%(value)s">%(player_b_name)s right?</a><br>
<a href="peril.cgi?action=c_correct&row=%(row)s&col=%(col)s&value=%(value)s">%(player_c_name)s right?</a>
</td>

<td width="40%%" align=center>
<a href="peril.cgi?action=done&row=%(row)s&col=%(col)s">done</td>

<td width="30%%">
<a href="peril.cgi?action=a_wrong&row=%(row)s&col=%(col)s&value=%(value)s">%(player_a_name)s wrong?</a></br>
<a href="peril.cgi?action=b_wrong&row=%(row)s&col=%(col)s&value=%(value)s">%(player_b_name)s wrong?</a><br>
<a href="peril.cgi?action=c_wrong&row=%(row)s&col=%(col)s&value=%(value)s">%(player_c_name)s wrong?</a>
</td>

</tr></table>
""" % data


######################################################

def html_start(player_view):
	if player_view=="auto":
		refresh_str = '<meta http-equiv="refresh" content="2">' 
	else:
		refresh_str = ''

	print """
<HTML>
<HEAD>
<TITLE>Peril</TITLE>
%s
</HEAD>
<BODY BGCOLOR="LightBlue">
""" % refresh_str

######################################################

player_view = 0

def html_end():
	global player_view

	if not player_view:
		print """
<p><table width="100%%" border=1><tr>
<td align=left><a href="peril.cgi">main</a></td>
<td><a href="peril.cgi?player_view=1">player view</a>
<a href="peril.cgi?player_view=auto">auto view</a></td>
<td><a href="peril.cgi?action=next">next round</a></td>
<td><a href="peril.cgi?action=undo">undo</a></td>
<td><a href="peril.cgi?action=show_form">edit game data</a></td>
<td><a href="peril.cgi?action=reset">reset</a></td>
</tr></table>"""
	print """</BODY></HTML>"""

######################################################

def right_answer(data, form, player):
	round = int(data["round"])
	# add question value to player's score
	score_str = "score_"+player
	score = int(data[score_str])
	row = int(form["row"].value)
	col = int(form["col"].value)
	value = int(form["value"].value)
	score += value
	data[score_str] = str(score)

	# mark question as answered
	index = "cat_%d_done" % col
	done = data[index]
	done = done[:row]+"1"+done[row+1:]
	data[index] = done

	# go back to showing board
	data["answer_to_show"] = "0"
	data["wager"] = "0"
	write_game_data(data)

######################################################

def wrong_answer(data, form, player):
	round = int(data["round"])
	# subtract question value from player's score
	score_str = "score_"+player
	score = int(data[score_str])
	row = form["row"].value
	col = form["col"].value
	value = int(form["value"].value)
	score -= value
	data[score_str] = str(score)

	write_game_data(data)

######################################################

def done(data, form):
	row = int(form["row"].value)
	col = int(form["col"].value)

	# mark question as answered
	index = "cat_%d_done" % col
	done = data[index]
	done = done[:row]+"1"+done[row+1:]
	data[index] = done

	# erase wager, if any
	data["wager"] = "0"

	# go back to showing board
	data["answer_to_show"] = "0"
	write_game_data(data)


######################################################

def do_action(action, data, form):
	round = int(data["round"])
	print "action=<b>%s</b><br>" % action
	if action=="none":
		return 
	if action=="show_answer":
		row = form["row"].value
		col = form["col"].value
		data["answer_to_show"] = "%s:%s" % (row, col)
		write_game_data(data)
		return

	if action=="a_correct":
		right_answer(data, form, "a")
		return
	if action=="b_correct":
		right_answer(data, form, "b")
		return
	if action=="c_correct":
		right_answer(data, form, "c")
		return
	if action=="a_wrong":
		wrong_answer(data, form, "a")
		return
	if action=="b_wrong":
		wrong_answer(data, form, "b")
		return
	if action=="c_wrong":
		wrong_answer(data, form, "c")
		return
	if action=="done":
		done(data, form)
		return

	if action=="set_super_wager":
		row = form["row"].value
		col = form["col"].value
		wager = int(form["wager"].value)
		data["wager"] = wager
		write_game_data(data)
		return

	if action=="reset":
		print """<h1>### RESET ###</h1>
Are you sure you want to reset the game?<br>
If so, click on the link below to really reset the game:<br>
<a href="peril.cgi?action=really_reset">Really Reset!!</a>
"""
		return

	if action=="really_reset":
		# save some variables...
		player_a_name = data["player_a_name"]
		player_b_name = data["player_b_name"]
		player_c_name = data["player_c_name"]

		reset_data = stub_data.copy()

		reset_data["player_a_name"] = player_a_name
		reset_data["player_b_name"] = player_b_name
		reset_data["player_c_name"] = player_c_name

		# remove all undo sequence files
		remove_undo_data_files()

		write_game_data(reset_data)
		data.update(reset_data)

		return

	if action=="next":
		round += 1
		data["round"] = str(round)

		# reset the "done" flags
		for i in range(0,6):
			data["cat_%d_done" % i] = "000000"

		write_game_data(data)
		return

	if action=="show_form":
		# show a form to set values directly
		# bank, score_a, score_b, round, strike_1,2,3
		print "Please edit the game data:"
		print """<FORM method=post action="peril.cgi">
<INPUT type=hidden name="action" value="set_values">
<table border=0>
"""
		keys = data.keys()
		keys.sort()
		for name in keys:
			value = data[name]
			print """
<tr>
<TD BGCOLOR="#dddddd" WIDTH="55%%">
%s</td>
<TD WIDTH="33%%"><INPUT type="Text" name="%s" size="50" value="%s">
</TD>
</tr>""" % (name, name, value)

		print """
</table>
<ul>
<input type="Submit" name="submit" value="Submit">
</ul>"""
		return

	if action=="set_values":
		# set values from the form
		for name in data.keys():
			value = form[name].value
			data[name] = value
		print "Set values for game from form data!!"
		write_game_data(data)
		return

	if action=="final_form":
		print "Please enter final round game data:"
		print """<FORM method=post action="peril.cgi">
<INPUT type=hidden name="action" value="set_final_values">
<table border=0>
"""
		for player in ["a", "b", "c"]:
			for field in ["question", "wager",]:
				print """
<tr>
<TD BGCOLOR="#dddddd" WIDTH="55%%">
%s_%s</td>
<TD WIDTH="33%%"><INPUT type="Text" name="%s_%s" size="50">
<INPUT type=hidden name="hide_%s_%s" value="1">
</TD>
</tr>""" % (field, player, field, player, field, player)
			

		print """
</table>
<ul>
<input type="Submit" name="submit" value="Submit">
</ul>"""
		return

	if action=="set_final_values":
		# set values from the form
		for player in ["a", "b", "c"]:
			for field in ["wager", "question"]:
				name = field+"_"+player
				value = form[name].value
				data[name] = value
				name = "hide_"+field+"_"+player
				value = form[name].value
				data[name] = value
			data["question_result_"+player] = "unknown"
			data["last_score_recorded_"+player] = "0"
		# increment round
		round += 1
		data["round"] = str(round)

		print "Set final values for game from form data!!"
		write_game_data(data)
		return

	if action=="reveal":
		field = form["field"].value
		player = form["player"].value
		data["hide_"+field+"_"+player] = "0"
		write_game_data(data)
		return

	if action=="set_last_question_result":
		player = form["player"].value
		result = form["result"].value
		data["question_result_"+player] = result
		data["show_final_correct_form_"+player] = "0"
		write_game_data(data)
		return

	if action=="last_score":
		player = form["player"].value
		wager = int(data["wager_"+player])
		score = int(data["score_"+player])
		result = data["question_result_"+player]
		if result=="correct":
			score = score + wager
		else:
			score = score - wager
			if score<0:
				score = 0
		data["score_"+player] = str(score)
		data["last_score_recorded_"+player] = "1"
		write_game_data(data)
		return

	if action=="undo":
		print """<h1>### UNDO ###</h1>
Are you sure you want to undo a step in the game?<br>
If so, click on the link below to really undo 1 game step:<br>
<a href="peril.cgi?action=really_undo">Really undo!!</a>
"""
		return

	if action=="really_undo":
		# back up one data file
		game_filename = data["game_filename"]
		os.unlink(game_filename)
		new_data = read_game_data_from_last_file()
		
		# replace items in data with items in new_data
		for key in data.keys():
			del(data[key])
		
		data.update(new_data)
		return

	print "-- Error: unknown action %s --" % action

	# end of do_action()

######################################################

if __name__ == "__main__":
	print "Content-type: text/html"
	print ""
	form = cgi.FieldStorage()

	player_view = 0
	if "player_view" in form:
		player_view = form["player_view"].value

	html_start(player_view)
	data = read_game_data_from_last_file()

	# use this for debugging
	if print_data and not player_view:
		print "data=", data

	if "action" in form:
		#print "form action value =\n<pre>", form["action"]
		#print "</pre>", form["action"].value
		action = form["action"].value
		do_action(action, data, form)
		
		# do_action doesn't return if it handled the action completely

	show_board(data, player_view)
	html_end()
