#!/usr/bin/env python

import sys, os, cgi

import cgitb ; cgitb.enable()

data_dir = "/home/game/wdata/"
game_filename = data_dir + "wheeldata.txt"

puzzle_size = 52
puzzle_mask_all = puzzle_size * "#"

# import the answer data
sys.path.append(data_dir)
from puzzles import *

stub_data = {
"score_a":"0",
"score_b":"0",
"score_c":"0",
"pot_a":"0",
"pot_b":"0",
"pot_c":"0",
"player_a_name":"Player A",
"player_b_name":"Player B",
"player_c_name":"Player C",
"wheel_pos":"0",
"cur_value":"0",
"cur_player":"a",
"cur_puzzle":"0",
"puzzle_mask":puzzle_mask_all,
}

# here is some display data
display_data = {
	"border":0,
	"debug":0,
	"size":5,
	"tsize":7,
	# main gameboard offsets
	"w0":30,
	"h0":30,
	# main gameboard cell dimensions
	"w":200,
	#"h":80,
	"h":40,

	# puzzle stuff
	"puzzle_offset": "300",
	"letter_size": "5",
	"letter_height": "60",
	"letter_width": "35",

	# score table stuff
	"player_a_color":"LightBlue",
	"player_b_color":"LightBlue",
	"player_c_color":"LightBlue",
	"cur_player_color":"yellow",
	"score_offset":100,
	"score_spacing":100,
	"score_size":200,
	"score_vspace":20,


	# wheel stuff
	"wslot_offset":100,
	"wslot_size":80,
}

# wheel tuple has (value or action, text color, color)
wheel_segments = [
("100", "black", "red"),
("250", "black", "orange"),
("400", "black", "yellow"),
("Bankrupt", "white", "black"),
("1000", "black", "gold"),
("300", "white", "green"),
("200", "white", "purple"),
("Lose turn", "white", "grey"),
("500", "black", "white"),
]


######################################################

def read_game_data(game_filename):
	game_lines = open(game_filename, "r").readlines()
	#game_lines=[]
	data = stub_data.copy()
	for line in game_lines:
		line = line.strip()
		if line and line[0]!='#':
			(name, value) = line.split('=',1)
			#print "data[%s]=%s<br>" % (name, value)
			data[name] = value
	data["game_filename"] = game_filename
	return data

######################################################

def write_game_data(data):
	game_filename = data["game_filename"]
	fd = open(game_filename, "w")
	klist = data.keys()
	klist.sort()
	for name in klist:
		fd.write("%s=%s\n" % (name, data[name]))
	fd.close()

######################################################



def show_wheel(data):
	# FIXTHIS - for now, show wheel as a simple line
	wpos = int(data["wheel_pos"])

	# set up values for wheel display
	for slot in range(0,len(wheel_segments)):
		segment = wpos - 4 + slot
		if segment<0:
			segment += len(wheel_segments)
		if segment>=len(wheel_segments):
			segment -= len(wheel_segments) 

		segment_data = wheel_segments[segment]
		data["slot_%d_text" % slot] = "&nbsp;"+segment_data[0]+"&nbsp;"
		data["slot_%d_tcolor" % slot] = segment_data[1]
		data["slot_%d_color" % slot] = segment_data[2]
	
	print """
letter_value = %(letter_value)s
<table cellspacing=0 cellpadding=0>
<tr>
<td width="%(wslot_offset)s"></td>
<td width="%(wslot_size)s"></td>
<td width="%(wslot_size)s"></td>
<td width="%(wslot_size)s"></td>
<td width="%(wslot_size)s"></td>
<td align="center" width="%(wslot_size)s">V</td>
<td width="%(wslot_size)s"></td>
<td width="%(wslot_size)s"></td>
<td width="%(wslot_size)s"></td>
<td width="%(wslot_size)s"></td>
<tr>
<td></td>
<td align="center" bgcolor="%(slot_0_color)s">
<font color="%(slot_0_tcolor)s">%(slot_0_text)s</font>
</td>
<td align="center" bgcolor="%(slot_1_color)s">
<font color="%(slot_1_tcolor)s">%(slot_1_text)s</font>
</td>
<td align="center" bgcolor="%(slot_2_color)s">
<font color="%(slot_2_tcolor)s">%(slot_2_text)s</font>
</td>
<td align="center" bgcolor="%(slot_3_color)s">
<font color="%(slot_3_tcolor)s">%(slot_3_text)s</font>
</td>
<td align="center" bgcolor="%(slot_4_color)s">
<font color="%(slot_4_tcolor)s">%(slot_4_text)s</font>
</td>
<td align="center" bgcolor="%(slot_5_color)s">
<font color="%(slot_5_tcolor)s">%(slot_5_text)s</font>
</td>
<td align="center" bgcolor="%(slot_6_color)s">
<font color="%(slot_6_tcolor)s">%(slot_6_text)s</font>
</td>
<td align="center" bgcolor="%(slot_7_color)s">
<font color="%(slot_7_tcolor)s">%(slot_7_text)s</font>
</td>
<td align="center" bgcolor="%(slot_8_color)s">
<font color="%(slot_8_tcolor)s">%(slot_8_text)s</font>
</td>

</tr>
</table>
""" % data

	# remove temp data
	for slot in range(0,len(wheel_segments)):
		del data["slot_%s_text" % slot]
		del data["slot_%s_tcolor" % slot]
		del data["slot_%s_color" % slot]

def show_puzzle(data):
	puz_index = int(data["cur_puzzle"])
	puzzle = puzzles[puz_index]
	ptext = puzzle[0]
	ptype = puzzle[1]
	pmask = data["puzzle_mask"]

	for i in range(puzzle_size):
		letter = ptext[i]
		mask = pmask[i]
		if letter==".":
			color = "green"
			contents = "&nbsp;"
		else:
			color = "white"
			if mask=="#":
				# show link to reveal letter
				contents = '<a href="/game-cgi/wheel.cgi?action=show_letter&letter_index=%d">&nbsp;</a>' % i
			else:
				contents = "<font size=%s><b>" % data["letter_size"]+letter+"</b></font>"
		lblock = '<table border=1 width="100%%" bgcolor="%s"><tr><td width="%s" height="%s" align="center">%s</td></tr></table>' % (color, data["letter_width"], data["letter_height"], contents)
		
		data["letter_%d" % i] = lblock
	print """
<table border=0 bgcolor="lightBlue" cellspacing=4 cellpadding=0>
<tr>
<td width="%(puzzle_offset)s></td>
<td width="%(letter_width)s"></td>
<td width="%(letter_width)s"></td>
<td width="%(letter_width)s"></td>
<td width="%(letter_width)s"></td>
<td width="%(letter_width)s"></td>
<td width="%(letter_width)s"></td>
<td width="%(letter_width)s"></td>
<td width="%(letter_width)s"></td>
<td width="%(letter_width)s"></td>
<td width="%(letter_width)s"></td>
<td width="%(letter_width)s"></td>
<td width="%(letter_width)s"></td>
<td width="%(letter_width)s"></td>
<td width="%(letter_width)s"></td>
<td width="%(letter_width)s"></td>
</tr>
<tr height="%(letter_height)s">
<td></td>
<td></td>
<td>%(letter_0)s</td>
<td>%(letter_1)s</td>
<td>%(letter_2)s</td>
<td>%(letter_3)s</td>
<td>%(letter_4)s</td>
<td>%(letter_5)s</td>
<td>%(letter_6)s</td>
<td>%(letter_7)s</td>
<td>%(letter_8)s</td>
<td>%(letter_9)s</td>
<td>%(letter_10)s</td>
<td>%(letter_11)s</td>
<td></td>
</tr>
<tr height="%(letter_height)s">
<td></td>
<td>%(letter_12)s</td>
<td>%(letter_13)s</td>
<td>%(letter_14)s</td>
<td>%(letter_15)s</td>
<td>%(letter_16)s</td>
<td>%(letter_17)s</td>
<td>%(letter_18)s</td>
<td>%(letter_19)s</td>
<td>%(letter_20)s</td>
<td>%(letter_21)s</td>
<td>%(letter_22)s</td>
<td>%(letter_23)s</td>
<td>%(letter_24)s</td>
<td>%(letter_25)s</td>
</tr>
<tr height="%(letter_height)s">
<td></td>
<td>%(letter_26)s</td>
<td>%(letter_27)s</td>
<td>%(letter_28)s</td>
<td>%(letter_29)s</td>
<td>%(letter_30)s</td>
<td>%(letter_31)s</td>
<td>%(letter_32)s</td>
<td>%(letter_33)s</td>
<td>%(letter_34)s</td>
<td>%(letter_35)s</td>
<td>%(letter_36)s</td>
<td>%(letter_37)s</td>
<td>%(letter_38)s</td>
<td>%(letter_39)s</td>
</tr>
<tr height="%(letter_height)s">
<td></td>
<td></td>
<td>%(letter_40)s</td>
<td>%(letter_41)s</td>
<td>%(letter_42)s</td>
<td>%(letter_43)s</td>
<td>%(letter_44)s</td>
<td>%(letter_45)s</td>
<td>%(letter_46)s</td>
<td>%(letter_47)s</td>
<td>%(letter_48)s</td>
<td>%(letter_49)s</td>
<td>%(letter_50)s</td>
<td>%(letter_51)s</td>
<td></td>
</tr>
</table>
</font>
""" % data

	print "<br><center>(%s)</center><br><hr>" % ptype

# show the game board
def show_board(data):
	player = data["cur_player"]

	# get size data for display
	data.update(display_data)

	# set current player color
	data["player_%s_color" % player] = data["cur_player_color"]

	# offset table in browser frame, and establish horizontal spacings
	print """
<table border=%(border)s bgcolor="orange" cellspacing=0 cellpadding=0>
<tr bgcolor="LightBlue">
<td width="%(w0)s" rowspan=8></td>
<td height="%(h0)s" width="%(w)s"></td>
<td width="%(w)s"></td>
<td width="%(w)s"></td>
<td width="%(w)s"></td>
<td width="%(w)s"></td>
<td width="%(w)s"></td>
<td width="%(w0)s" rowspan=8></td>
</tr>""" % data

	# title
	title = "Wheel of Treasure"
	data["title"] = title
	print """
<tr bgcolor="LightBlue">
<td colspan=7 align=center><b><font size="%(tsize)s" color="crimson">
%(title)s</font></font></td>
</tr>""" % data

	# show round and multiplier information
	print "</font></td></tr>" % data

	# use this to debug columns
	if data["debug"]==1:
		print """
<tr bgcolor="LightBlue">
<td align=center>1</td>
<td align=center>2</td>
<td align=center>3</td>
<td align=center>4</td>
<td align=center>5</td>
<td align=center>6</td>
</tr>""" % data
	
	# end of main gameboard table
	print "</table>"

	# show the puzzle
	show_puzzle(data)


	# scores and pots
	print"""
<table border=%(border)s bgcolor="LightBlue" cellspacing=0 cellpadding=0>
<tr bgcolor="LightBlue">
<td width="%(score_offset)s"></td>
<td align=center width="%(score_size)s" bgcolor="%(player_a_color)s">
  <font size=%(size)s><b>%(player_a_name)s</b></font>
</td>
<td width="%(score_spacing)s"></td>
<td align=center width="%(score_size)s" bgcolor="%(player_b_color)s">
  <font size=%(size)s><b>%(player_b_name)s</b></font>
</td>
<td width="%(score_spacing)s"></td>
<td align=center width="%(score_size)s" bgcolor="%(player_c_color)s">
  <font size=%(size)s><b>%(player_c_name)s</b></font>
</td>
</tr>
<tr>
<td>Score:</td>
<td align="center" bgcolor="%(player_a_color)s">
  <table border=1 bordercolor=black><tr><td align=right bgcolor=white>
      <font size=%(size)s>&nbsp;&nbsp;%(score_a)s&nbsp;&nbsp;</font>
  </table>
</td>
<td></td>
<td align="center" bgcolor="%(player_b_color)s">
  <table border=1 bordercolor=black><tr><td align=right bgcolor=white>
      <font size=%(size)s>&nbsp;&nbsp;%(score_b)s&nbsp;&nbsp;</font>
  </table>
</td>
<td></td>
<td align="center" bgcolor="%(player_c_color)s">
  <table border=1 bordercolor=black><tr><td align=right bgcolor=white>
      <font size=%(size)s>&nbsp;&nbsp;%(score_c)s&nbsp;&nbsp;</font>
  </table>
</td>
</tr>
<tr height="%(score_vspace)s">
<td></td>
<td bgcolor="%(player_a_color)s"></td>
<td></td>
<td bgcolor="%(player_b_color)s"></td>
<td></td>
<td bgcolor="%(player_c_color)s"></td>
</tr>
<tr>
<td>Pot:</td>
<td align="center" bgcolor="%(player_a_color)s">
  <table border=1 bordercolor=black><tr><td align=right bgcolor=white>
	<font size=%(size)s>&nbsp;&nbsp;%(pot_a)s&nbsp;&nbsp;</font></td></tr>
  </table>
</td>
<td></td>
<td align="center" bgcolor="%(player_b_color)s">
  <table border=1 bordercolor=black><tr><td align=right bgcolor=white>
	<font size=%(size)s>&nbsp;&nbsp;%(pot_b)s&nbsp;&nbsp;</font></td></tr>
  </table>
</td>
<td></td>
<td align="center" bgcolor="%(player_c_color)s">
  <table border=1 bordercolor=black><tr><td align=right bgcolor=white>
	<font size=%(size)s>&nbsp;&nbsp;%(pot_c)s&nbsp;&nbsp;</font></td></tr>
  </table>
</td>
</tr>
<tr height="%(score_vspace)s">
<td></td>
<td bgcolor="%(player_a_color)s"></td>
<td></td>
<td bgcolor="%(player_b_color)s"></td>
<td></td>
<td bgcolor="%(player_c_color)s"></td>
</tr>
</table>""" % data

	# show the actual wheel
	show_wheel(data)

	# clean up data
	for key in display_data.keys():
		del data[key]

	# end of show_board()

######################################################

def html_start(spin_left):
	print """
<HTML>
<head>
<TITLE>Wheel</TITLE>"""
	if spin_left:
		import random
		spin_left = spin_left-(random.randint(1,3))
		if spin_left < 0:
			spin_left = 0
		timeout = 500
		# slow down at end
		if spin_left>4:
			timeout = 1000
		elif spin_left>1:
			timeout = 2000

		print """
<script language="JavaScript">
<!--
var time = null
function move() {
window.location = 'http://www.freeonlinecharts.com/game-cgi/wheel.cgi?action=spin&spin_left=%d'
}
//-->
</script>
</head>
<BODY BGCOLOR="LightBlue" onload="timer=setTimeout('move()',%s)">
""" % (spin_left, timeout)
	else:
		print """
</head>
<BODY BGCOLOR="LightBlue">
"""

######################################################
# set spin_left to 0, to move wheel by 1 segment (for debugging)

def html_end():
	print """
<p><table width="100%%" border=1>
<tr>
<td align="center"><a href="wheel.cgi">main</a></td>
<td align="center"><a href="wheel.cgi?action=spin&spin_left=18">spin</a></td>
<td align="center"><a href="wheel.cgi?action=guess_wrong">guess wrong</a></td>
<td align="center"><a href="wheel.cgi?action=puzzle_solved">puzzle solved</a></td>
</tr>
<tr>
<td align="center"><a href="wheel.cgi?action=buy_vowel">buy_vowel</a></td>
<td align="center"><a href="wheel.cgi?action=next_puzzle">next puzzle</a></td>
<td align="center"><a href="wheel.cgi?action=switch_player">switch player</a></td>
<td align="center"><a href="wheel.cgi?action=show_form">edit game data</a></td>
<td align="center"><a href="wheel.cgi?action=reset">reset</a></td>
</table>
</BODY>
</HTML>"""

######################################################

def do_action(action, data, form):
	player = data["cur_player"]
	#print "action=<b>%s</b><br>" % action
	if action=="none":
		return

	if action=="spin":
		spin_left = int(form["spin_left"].value)

		# move wheel position
		wheel_pos = int(data["wheel_pos"])
		wheel_pos -= 1
		if wheel_pos >= len(wheel_segments):
			wheel_pos = 0
		if wheel_pos < 0:
			wheel_pos = len(wheel_segments)-1
		data["wheel_pos"] = str(wheel_pos)
		write_game_data(data)

		# if wheel stopped, perform action for this segment
		if spin_left>0:
			return

		segment_data = wheel_segments[wheel_pos]
		value = segment_data[0]
		pstr = "pot_%s" % player

		try:
			data["letter_value"] = str(int(value))
			write_game_data(data)
			return
		except:
			if value=="Bankrupt":
				data[pstr] = "0"
				write_game_data(data)
				action = "switch_player"
			elif value=="Lose Turn":
				action = "switch_player"
			else:
				print "UNKNOWN special segment!!"
				return

		# fall through to another action

	if action=="show_letter":
		# add wheel value to player's pot
		letter_value = int(data["letter_value"])
		pstr = "pot_%s" % player
		try:
			#value = int(segment_data[0])
			old_pot = int(data[pstr])
			data[pstr] = str(old_pot + letter_value)
			write_game_data(data)
		except:
			pass

		# reveal letter
		letter_index = int(form["letter_index"].value)
		pmask = data["puzzle_mask"]
		pmask = pmask[:letter_index]+" "+pmask[letter_index+1:]
		data["puzzle_mask"] = pmask
		write_game_data(data)
		
		return

	if action=="buy_vowel":
		# set letter_value to -250
		data["letter_value"] = "-250"
		write_game_data(data)
		return

	if action=="guess_wrong":
		# switch player
		action = "switch_player"
		# drop through to other 'if'

	if action=="puzzle_solved":
		# add player pot to player score
		pot_value = int(data["pot_%s" % player])
		score_str = "score_%s" % player
		score = int(data[score_str]) + pot_value
		data[score_str] = str(score)

		# clear all pots to 0
		for p in ["a", "b", "c"]:
			data["pot_%s" % p] = "0"

		# clear letter value
		data["letter_value"] = "0"
		# reveal puzzle
		data["puzzle_mask"] = puzzle_size * " "
		write_game_data(data)
		return

	if action=="switch_player":
		if player=="a":
			player="b"
		elif player=="b":
			player="c"
		else:
			player="a"

		data["cur_player"] = player
		write_game_data(data)
		return
		
	if action=="done":
		done(data, form)
		return

	if action=="reset":
		print """<h1>### RESET ###</h1>
Are you sure you want to reset the game?<br>
If so, click on the link below to really reset the game:<br>
<a href="wheel.cgi?action=really_reset">Really Reset!!</a>
"""
		return

	if action=="really_reset":
		data["score_a"]="0"
		data["score_b"]="0"
		data["score_c"]="0"
		data["pot_a"]="0"
		data["pot_b"]="0"
		data["pot_c"]="0"
		data["cur_puzzle"]="0"
		data["puzzle_mask"]= puzzle_mask_all
		data["letter_value"]= "0"
		write_game_data(data)
		return

	if action=="show_form":
		# show a form to set values directly
		# bank, score_a, score_b, round, strike_1,2,3
		print "Please edit the game data:"
		print """<FORM method=post action="wheel.cgi?action=set_values">
<INPUT type=hidden name="action" value="set_values">
<table border=0>
"""
		keys = data.keys()
		keys.sort()
		for name in keys:
			value = data[name]
			print """
<tr>
<TD BGCOLOR="#dddddd" WIDTH="55%%">
%s</td>
<TD WIDTH="33%%"><INPUT type="Text" name="%s" size="50" value="%s">
</TD>
</tr>""" % (name, name, value)

		print """
</table>
<ul>
<input type="Submit" name="submit" value="Submit">
</ul>"""
		return

	if action=="set_values":
		# set values from the form
		for name in data.keys():
			value = form[name].value
			data[name] = value
		print "Set values for game from form data!!"
		write_game_data(data)
		return

	if action=="final_form":
		print "Please enter final round game data:"
		print """<FORM method=post action="wheel.cgi?action=set_final_values">
<INPUT type=hidden name="action" value="set_final_values">
<table border=0>
"""
		for player in ["a", "b", "c"]:
			for field in ["wager", "answer"]:
				print """
<tr>
<TD BGCOLOR="#dddddd" WIDTH="55%%">
%s_%s</td>
<TD WIDTH="33%%"><INPUT type="Text" name="%s_%s" size="50">
<INPUT type=hidden name="hide_%s_%s" value="1">
</TD>
</tr>""" % (field, player, field, player, field, player)
			

		print """
</table>
<ul>
<input type="Submit" name="submit" value="Submit">
</ul>"""
		return

	if action=="set_final_values":
		# set values from the form
		for player in ["a", "b", "c"]:
			for field in ["wager", "answer"]:
				name = field+"_"+player
				value = form[name].value
				data[name] = value
				name = "hide_"+field+"_"+player
				value = form[name].value
				data[name] = value
			data["show_final_correct_form_"+player] = "0"
		# increment round
		round += 1
		data["round"] = str(round)

		print "Set final values for game from form data!!"
		write_game_data(data)
		return

	if action=="next_puzzle":
		cur_puzzle = int(data["cur_puzzle"])
		if cur_puzzle < len(puzzles)-1:
			cur_puzzle += 1
			data["cur_puzzle"] = str(cur_puzzle)
			data["puzzle_mask"] = puzzle_mask_all
			write_game_data(data)
		else:
			print "Currently showing last puzzle."
		return

	if action=="final_result":
		player = form["player"].value
		result = form["result"].value
		wager = int(data["wager_"+player])
		score = int(data["score_"+player])
		if result=="correct":
			score += wager
		else:
			score -= wager
			if score < 0:
				score = 0
		data["score_"+player] = str(score)
		data["show_final_correct_form_"+player] = "0"
		write_game_data(data)
		return


	print "-- Error: unknown action %s --" % action

	# end of do_action()

######################################################

if __name__ == "__main__":
	print "Content-type: text/html"
	print ""
	form = cgi.FieldStorage()

	if "spin_left" in form:
		spin_left = form["spin_left"].value
		try:
			spin_left = int(spin_left)
		except:
			spin_left = 0
	else:
		spin_left = 0

	html_start(spin_left)
	data = read_game_data(game_filename)

	if "action" in form:
		action = form["action"].value
		do_action(action, data, form)
		
		# do_action doesn't return if it handled the action completely

	show_board(data)
	html_end()
