#!/usr/bin/env python

import sys, os, cgi

data_dir = "/home/survey/data/"
game_filename = data_dir + "gamedata.txt"

no_strike = "."

#format of answers list is: (
stub_data = {"bank":"0",
"score_a":"0",
"score_b":"0",
"round":"0",
"question":"nothing",
"strike_1":no_strike,
"strike_2":no_strike,
"strike_3":no_strike,
"team_a_name":"Team A",
"team_b_name":"Team B",
}

def get_cur_filename(data):
	global data_dir
	round = int(data["round"])
	try:
		if round<6:
			attr = "file_%d" % round
		elif round>=6 and round<=10:
			attr = "fast_%d" % (round-5)
		else:
			attr = "fast_%d" % (round-10)

		cur_filename = data_dir + data[attr]
	except:
		print "Error: value for %s not found in game data" % attr
		cur_filename = "error"
	return cur_filename

######################################################

# answer files must be of following format:
#8. Other than Linus Torvalds, name a famous Linux kernel developer:
#----------------------------------
#24:Alan Cox
#14:Andrew Morton
#10:Ingo Molnar
# 4:Greg Kroah-Hartman

def import_answers(data):
	answer_file = get_cur_filename(data)
	if answer_file=="error":
		print "answer_file=error"
		return

	try:
		answer_lines = open(answer_file, "r").readlines()
	except:
		print "Error: can't open answer file '%s'" % answer_file
		return
	num = 1
	# parse answers and set them in the data
	question = answer_lines[0].split(".", 1)[1]
	data["question"] = question.strip()
	for line in answer_lines[2:]:
		(value, answer) = line.split(':',1)
		#print "data[%s]=%s<br>" % (value, answer)
		data["value_%d" % num] = value
		data["answer_%d" % num] = answer.strip()
		num += 1
	# clear out remaining answers, if any
	while num<10:
		if data.has_key("answer_%d" % num):
			del data["answer_%d" % num]
			del data["value_%d" % num]
		num += 1

######################################################

def read_game_data(game_filename):
	game_lines = open(game_filename, "r").readlines()
	data = stub_data.copy()
	for line in game_lines:
		line = line.strip()
		if line and line[0]!='#':
			(name, value) = line.split('=',1)
			#print "data[%s]=%s<br>" % (name, value)
			data[name] = value
	data["game_filename"] = game_filename
	return data

######################################################

def write_game_data(data):
	game_filename = data["game_filename"]
	fd = open(game_filename, "w")
	klist = data.keys()
	klist.sort()
	for name in klist:
		fd.write("%s=%s\n" % (name, data[name]))
	fd.close()

######################################################

def print_answer(column, num, value, answer, state):
	if column==1:
		print "<tr>"
	size = data["size"]
	print """<td WIDTH="%s"><font size=%d>%d</td>""" % (data["a1"], size, num)
	if state=="hidden":
		if int(value)>0:
			value = "XX"
			answer = "XXXXXXXXXXXX"
		else:
			value = "--"
			answer = "------------------"
	print """<td WIDTH="%s" align=right>""" % data["a2"]
	print """<font size=%d>%s</font></td>""" % (size, value)
	print """<td WIDTH="%s"><font size=%d>%s</td>""" % (data["a3"], size, answer)
	if column==1:
		print """<td WIDTH="%s"></td>""" % data["a4"]
	if column==2:
		print "</tr>"

######################################################

def print_fast_answer(column, num, value, answer, state):
	if column==1:
		print "<tr>"
	size = data["size"]
	print """<td WIDTH="%s"><font size=%d color="DarkRed">%d</td>""" % (data["a1"], size, num)
	if state=="hidden":
		value = "XX"
		answer = "XXXXXXXXXXXXXX"
		if num>5:
			value = "--"
			answer = "------------------"
	elif state=="answer_visible":
		value="XX"
	
	print """<td WIDTH="%s" align=right>""" % data["a2"]
	print """<font size=%d color="DarkRed">%s</font></td>""" % (size, value)
	print """<td WIDTH="%s"><font size=%d color="DarkRed">%s</td>""" % (data["a3"], size, answer)
	if column==1:
		print """<td WIDTH="%s"></td>""" % data["a4"]
	if column==2:
		print "</tr>"

######################################################

# show the game board
# big_board indicates whether to use large fonts and fill the screen

# here is some size data
bigsize_data = {
	"border":0,
	"debug":0,
	"size":5,
	"tsize":7,
	"strike_width":40,
	"uc-height":180,
	"c-width":250,
	"lc-height":160,
	# main gameboard offsets
	"w0":30,
	"h0":30,
	# main gameboard column widths
	"w1":150,
	"w2":15,
	"w3":80,
	"w4":600,
	# answer table column widths
	"a1":30,
	"a2":50,
	"a3":300,
	"a4":20,
}

smallsize_data = {
	"border":0,
	"debug":0,
	"size":3,
	"tsize":3,
	"strike_width":20,
	"uc-height":70,
	"c-width":180,
	"lc-height":70,
	# main gameboard offsets
	"w0":1,
	"h0":1,
	# main gameboard column widths
	"w1":40,
	"w2":5,
	"w3":100,
	"w4":200,
	# answer table column widths
	"a1":25,
	"a2":40,
	"a3":400,
	"a4":20,
}

def show_game(data, big_board):
	round = int(data["round"])

	# get size data for display
	if big_board:
		size_data = bigsize_data
	else:
		size_data = smallsize_data
	data.update(size_data)

	# offset table in browser frame, and establish horizontal spacings
	print """
<table border=%(border)s bgcolor="orange" cellspacing=0 cellpadding=0>
<tr bgcolor="LightBlue">
<td width="%(w0)s" rowspan=8></td>
<td height="%(h0)s" width="%(w1)s"></td>
<td width="%(w2)s"></td>
<td width="%(w3)s"></td>
<td width="%(w4)s"></td>
<td width="%(w3)s"></td>
<td width="%(w2)s"></td>
<td width="%(w1)s"></td>
<td width="%(w0)s" rowspan=8></td>
</tr>""" % data

	# title
	print """
<tr bgcolor="LightBlue">
<td colspan=7 align=center><b><font size="%(tsize)s" color="crimson">
Forum Faceoff</font></font></td>
</tr>""" % data

	# bank
	print """
<tr>
<td colspan=3><image src="/~survey/lightblue-upper-left-corner.gif"
 height="%(uc-height)s" width="%(c-width)s" alt="ul-corner"></td>
<td align=center>
	<font size=%(size)s><B>Bank</B>
	<table border=1 bordercolor="black"><tr><td align=right bgcolor=white>
		&nbsp;&nbsp;<font size=%(size)s>%(bank)s&nbsp;&nbsp;</td><br>
	</table>
<br></td>
<td colspan=3><image src="/~survey/lightblue-upper-right-corner.gif"
 height="%(uc-height)s" width="%(c-width)s" alt="ur-corner"></td>
</tr>""" % data

	# use this to debug columns
	if data["debug"]==1:
		print """
<tr bgcolor="LightBlue">
<td align=center>1</td>
<td align=center>2</td>
<td align=center>3</td>
<td align=center>4</td>
<td align=center>5</td>
<td align=center>6</td>
<td align=center>7</td>
</tr>""" % data
	
	# Team A score
	print"""
<tr>
<td align=center><font size=%(size)s><b>%(team_a_name)s</b><br><br></font>
<table border=1 bordercolor=black><tr><td align=right bgcolor=white>
	<font size=%(size)s>&nbsp;&nbsp;%(score_a)s&nbsp;&nbsp;</font></td></tr>
</table></td>
<td></td>
<td colspan=3><table border=4 bgcolor=yellow>""" % data

	# central answer table
	if round<6:
		# print table for normal round
		for num in [1,5,2,6,3,7,4,8]:
			ns = str(num)
			try:
				value = data["value_"+ns]
				answer = data["answer_"+ns]
				state = data["state_"+ns]
			except:
				value = "-1"
				answer = "nothing"
				state = "hidden"
			if num<=4:
				column = 1
			else:
				column = 2
			#print "num=%d, value=%s, answer=%s<br>" % (num, value, answer)
			print_answer(column, num, value, answer, state) 
	else:
		# print table for fast round
		for num in [1,5,2,6,3,7,4,8]:
			ns = str(num)
			try:
				answer = data["fast_answer_"+ns]
			except:
				answer = "nothing"
			try:
				value = data["fast_value_"+ns]
			except:
				value = "-1"
			try:
				state = data["state_"+ns]
			except:
				state = "hidden"

			if num<=4:
				column = 1
			else:
				column = 2
			#print "num=%d, value=%s, answer=%s<br>" % (num, value, answer)
			print_fast_answer(column, num, value, answer, state) 

	# Team B score
	print """
</table></td>
<td></td>
<td align=center><font size=%(size)s><b>%(team_b_name)s</b><br><br></font>
<table border=1 bordercolor=black><tr><td align=right bgcolor=white>
	<font size=%(size)s>&nbsp;&nbsp;%(score_b)s&nbsp;&nbsp;</font></td></tr>
</table></td>
</tr>""" % data

	# strikes row
	for s in (1,2,3):
		if data["strike_%d" % s]==no_strike:
			data["strike_%d" % s]= "&nbsp;"
	print """
<tr>
<td colspan=3><image src="/~survey/lightblue-lower-left-corner.gif"
 height="%(lc-height)s" width="%(c-width)s" alt="ul-corner"></td>
<td align=center><table border=0><tr>
  <td align=center width=20%%><font size=%(size)s>strike<br>
	<table border=1 bordercolor="orange"><tr><td bgcolor="yellow"
	width="%(strike_width)s" align=center>
	<font size=%(size)s color="red">%(strike_1)s</font></td></tr>
	</table>
  </td>
  <td width=20%%>&nbsp;&nbsp;&nbsp;&nbsp;</td>
  <td align=center width=20%%><font size=%(size)s>strike<br>
	<table border=1 bordercolor="orange"><tr><td bgcolor="yellow"
	width="%(strike_width)s" align=center>
	<font size=%(size)s color="red">%(strike_2)s</td></tr>
	</table>
  </td>
  <td width=20%%>&nbsp;&nbsp;&nbsp;&nbsp;</td>
  <td align=center width=20%%><font size=%(size)s>strike<br>
	<table border=1 bordercolor="orange"><tr><td bgcolor="yellow"
	width="%(strike_width)s" align=center>
	<font size=%(size)s color="red">%(strike_3)s</td></tr>
	</table>
  </td>
</table></td>
<td colspan=3><image src="/~survey/lightblue-lower-right-corner.gif"
 height="%(lc-height)s" width="%(c-width)s" alt="ul-corner"></td>
</tr>""" % data

	# show round and multiplier information
	print """
<tr bgcolor="LightBlue">
<td colspan=7 align=center><font size="%(size)s">""" % data
	if round==1 or round==2:
		print "Round %(round)s" % data
	if round==3 or round==4:
		print "Round %(round)s: Point values are <b>DOUBLED</b>" % data
	if round==5:
		print "Round %(round)s: Point values are <b>TRIPLED</b>" % data
	if round>=6 and round<=10:
		print "Fast Round 1"
	if round>10:
		print "Fast Round 2"
	print "</font></td></tr>" % data

	# end of main gameboard table
	print "</table>"

	# clean up data
	for s in (1,2,3):
		if data["strike_%d" % s]=="&nbsp;":
			data["strike_%d" % s] = no_strike
	for key in size_data.keys():
		del data[key]

	# end of show_game()

######################################################

def html_start():
	print """
<HTML>
<TITLE>Forum Faceoff</TITLE>
<BODY BGCOLOR="LightBlue">
"""

######################################################

def html_end():
	print """
<table width=100%%><tr>
<td align=left><a href="game.cgi">.</a></td>
<td align=right><a href="game.cgi?action=none">;</a></td>
</tr></table>
</BODY>
</HTML>"""

######################################################

def show_control(action, data):
	round = int(data["round"])
	cur_file = get_cur_filename(data)
	try:
		print "<hr>Control game: Round %d : File=%s: Question= %s" % (round, os.path.basename(cur_file), data["question"])
	except:
		print "error in game data - missing question"
	print "<ul>"

	if round<6:
		# show "reveal" controls
		for num in 1,2,3,4,5,6,7,8:
			ns = str(num)
			try:
				value = data["value_"+ns]
				answer = data["answer_"+ns]
				state = data["state_"+ns]
			except:
				value = "-1"
				answer = "nothing"
				state = "hidden"
			#print "num=%d, value=%s, answer=%s<br>" % (num, value, answer)
			if state=="hidden" and int(value)>0:
				print """<li><a href="game.cgi?action=reveal_%d">reveal #%d. '%s' (for %s points)</a>""" % (num, num, answer, value)
	else:
		# show show_fast_answer control
		ans_num = round-5
		if ans_num>5:
			ans_num -= 5

		state = data["state_%d" % ans_num]
		need_fast_answers = 0
		try:
			fast_answer = data["fast_answer_%d" % ans_num]
		except:
			need_fast_answers = 1
		if state=="hidden" and need_fast_answers==0:
			print """<li><a href="game.cgi?action=show_fast_answer_%d">show_answer #%d. '%s'</a>""" % (ans_num, ans_num, fast_answer)

		if need_fast_answers==1:
			print """
<li><a href="game.cgi?action=show_fast_form">enter fast round answers</a>"""
			
		if state=="answer_visible":
			fast_answer = data["fast_answer_%d" % ans_num]
			# show "match" controls
			for num in 1,2,3,4,5,6,7,8:
				ns = str(num)
				try:
					value = data["value_"+ns]
					answer = data["answer_"+ns]
					state = data["state_"+ns]
				except:
					value = "-1"
					answer = "nothing"
					state = "hidden"
				#print "num=%d, value=%s, answer=%s<br>" % (num, value, answer)
				if int(value)>0:
					print """<li><a href="game.cgi?action=match_%d">match '%s' with #%d. '%s' (for %s points)</a>""" % (num, fast_answer, num, answer, value)

	if round<6:
		print """
<li><a href="game.cgi?action=strike">strike</a>
<li><a href="game.cgi?action=bank_to_a">bank to "%(team_a_name)s"</a>
<li><a href="game.cgi?action=bank_to_b">bank to "%(team_b_name)s"</a>""" % data

	# separate exception-handling controls from regular controls
	print "</ul><hr><ul>"


	print """
<li><a href="game.cgi?action=next">next round</a>"""

	if action=="reset":
		print """<li><a href="game.cgi?action=really_reset">Really reset game??? Are you sure?</a>"""
	else:
		print """<li><a href="game.cgi?action=reset">reset game</a>"""

	print """<li><a href="game.cgi?action=show_form">edit game data directly</a>"""
	print """<li><a href="game.cgi?action=none">do nothing (refresh controls)</a>"""
	print "</ul>"
	print """<a href="game.cgi">Just show the game (without control links)</a>"""

	# end of show_control()

######################################################

def do_action(action, data, form):
	round = int(data["round"])
	print "action=<b>%s</b><br>" % action
	if action=="none":
		return
	if action=="bank_to_a":
		# add bank to team a
		score_a = int(data["score_a"])
		score_a += int(data["bank"])
		data["score_a"] = str(score_a)
		data["bank"] = "0"
		write_game_data(data)
		return
	if action=="bank_to_b":
		# add bank to team b
		score_b = int(data["score_b"])
		score_b += int(data["bank"])
		data["score_b"] = str(score_b)
		data["bank"] = "0"
		write_game_data(data)
		return
	if action=="reset":
		print "Are you sure?  If so, click on the link below to really reset the game"
		return
	if action=="really_reset":
		data["score_a"]="0"
		data["score_b"]="0"
		data["round"]="0"
		round = 0

	if action=="next" or action=="really_reset":
		# reset strikes, increment question, mark all answers hidden
		for i in 1,2,3:
			data["strike_%d" % i] = no_strike
		round += 1
		data["round"] = str(round)
		if round<6 or action=="really_reset":
			for a in range(1,9):
				data["state_%d" % a] = "hidden"
			data["bank"] = "0"
		# on round 6 and 11, get ready for fast rounds
		if round==6 or round==11:
			for i in range(1,6):
				try:
					del data["fast_answer_%d" % i]
				except:
					pass
				try:
					del data["fast_value_%d" % i]
				except:
					pass
			for a in range(1,9):
				data["state_%d" % a] = "hidden"

		# import data for this question
		import_answers(data)

		write_game_data(data)
		return
	if action=="strike":
		# change next strike to X
		for s in 1,2,3:
			strike = data["strike_%d" % s]
			if strike==no_strike:
				data["strike_%d" % s] = "X"
				break
		write_game_data(data)
		return
	if action.startswith("reveal"):
		# add value to bank, mark answer visible
		rnum = action[7:]
		state = data["state_%s" % rnum]
		if state != "hidden":
			print "Error - answer # %s is already revealed" % rnum
			return
		data["state_%s" % rnum] = "visible"
		value = int(data["value_%s" % rnum])
		multiplier = 1
		if data["round"]=="3" or data["round"]=="4":
			multiplier = 2
		if data["round"]=="5":
			multiplier = 3
		data["bank"] = int(data["bank"])+(value * multiplier)
		write_game_data(data)
		return

	if action.startswith("show_fast_answer"):
		ans_num = action.split("_")[3]
		data["state_%s" % ans_num] = "answer_visible"
		write_game_data(data)
		return
		
	if action.startswith("match"):
		# record value of answer, mark answer as visible
		rnum = action[6:]
		value = int(data["value_%s" % rnum])
		
		round = int(data["round"])
		ans_num = round-5
		if ans_num>5:
			 ans_num -= 5

		data["fast_value_%s" % ans_num] =  str(value)

		state = data["state_%d" % ans_num]
		if state == "visible":
			print "Error - answer # %s is already revealed" % ans_num
			return
		if state == "hidden":
			print "Error - answer # %s has not been revealed" % ans_num
			return
		data["state_%s" % ans_num] = "visible"
		data["bank"] = int(data["bank"])+int(value)
		write_game_data(data)
		print "done"
		return


	if action=="show_form":
		# show a form to set values directly
		# bank, score_a, score_b, round, strike_1,2,3
		print "Please edit the game data:"
		print """<FORM method=post action="/survey-cgi/game.cgi?action=set_values">
<INPUT type=hidden name="action" value="set_values">
<table border=0>
"""
		keys = data.keys()
		keys.sort()
		for name in keys:
			value = data[name]
			print """
<tr>
<TD BGCOLOR="#dddddd" WIDTH="55%%">
%s</td>
<TD WIDTH="33%%"><INPUT type="Text" name="%s" size="50" value="%s">
</TD>
</tr>""" % (name, name, value)

		print """
</table>
<ul>
<input type="Submit" name="submit" value="Submit">
</ul>"""
		return

	if action=="set_values":
		# set values from the form
		for name in data.keys():
			value = form[name].value
			data[name] = value
		print "Set values for game from form data!!"
		write_game_data(data)
		return

	if action=="show_fast_form":
		# collect question from files
		fast_questions = []
		for i in range(1,6):
			try:
				fast_file = data["fast_%d" % i]
				question = open(data_dir+fast_file, "r").readline()
				question = question.strip()
				question = question.split(".",1)[1]
			except:
				question = "fast round answer %d (error: couldn't read file %s)" % (i, fast_file)
			fast_questions.append(question)
	
		# show a form to enter fast round answers
		print "Please enter the fast round answers:"
		print """<FORM method=post action="/survey-cgi/game.cgi?action=set_fast_values">
<INPUT type=hidden name="action" value="set_fast_values">
<table border=0>
"""
		for num in range(1,6):
			print """
<tr>
<TD BGCOLOR="#dddddd" WIDTH="55%%">
%d. %s</td>
<TD WIDTH="33%%"><INPUT type="Text" name="fast_answer_%d" size="50" value="">
</TD>
</tr>""" % (num, fast_questions[num-1], num)
		print """
</table>
<ul>
<input type="Submit" name="submit" value="Submit">
</ul>"""
		return
	
	if action=="set_fast_values":
		# set values from the form
		try:
			for num in range(1,6):
				name = "fast_answer_%d" % num
				value = form[name].value
				data[name] = value
				data["state_%d" % num] = "hidden"
		except:
			print "couldn't read fast_answer data from form"
		print "Set fast values for game from form data!!"
		write_game_data(data)
		return

	if action=="hide_fast_answers":
		for i in range(1,6):
			data["state_%d" % i] = "hidden"
			del data["fast_answer_%d" % i]
		write_game_data(data)
		return
	
				
	print "-- Error: unknown action %s --" % action

	# end of do_action()

######################################################

if __name__ == "__main__":
	print "Content-type: text/html"
	print ""
	form = cgi.FieldStorage()

	html_start()

	try:
		data = read_game_data(game_filename)
	except:
		print """
			<h2 align=center>Game Error</h2>
			<p>
			<font color="red">Error: can't open game data file</font>"""
		sys.exit(1)

	if "action" not in form:
		show_game(data, 1)
		html_end()
		sys.exit(0)

	action = form["action"].value
	do_action(action, data, form)
	print "<hr>"
	show_game(data, 0)
	show_control(action, data)
	html_end()
